/**
 * @author: Romain Lovalente/ romain.lovalente-leherissier@isae.fr
 * **/


let SciviJS_GUI=require('scivijs_gui');
let SciviJS=SciviJS_GUI.SciviJS;
let PlugInBlock = SciviJS.PlugInBlock;



class myBlock1 extends PlugInBlock {

  /**
   * Constructor for yourBlock block
   * @param myFirstParameter : ...
   * @param ...
   * **/
  constructor (parentBlock, defFactor=1) {
    // Only initialize setters (maybe getters if you want), this,
    // and your custom parameters

    let setters = {
      'defFactor': (facteur) => {this._defFactorNode.number=facteur;}
    };

    super(parentBlock, setters);
    
    this.inputDataDim = 1;

    this._defFactor=defFactor;
    this.deformation= undefined;
    this._defFactorNode=undefined;
    this._deformVector= undefined;
  };

  /**
   * Method that initialize your node structures and geometries
   * **/
  _process () {
	 
	 this._setZWarpNode();
	 
	 this.addTransformNode('ADD',this.deformation);
    
  };
	
	_setZWarpNode(){
		
	 let zAxis = new THREE.Vector3Node(0,0,1);
	 
	 this._defFactorNode = new THREE.FloatNode(this._defFactor);
	 
	 this._deformVector= new THREE.OperatorNode( zAxis , this._defFactorNode,
	 THREE.OperatorNode.MUL);
		
		this.deformation = new THREE.OperatorNode(
			this._inputDataNode,this._deformVector,
			THREE.OperatorNode.MUL);
	}
	
	_setInput (dataName, componentNames) {
    super._setInput(dataName, componentNames);

    if (this._processed && this.deformation) {
		let oldNode = this.deformation;
		
		this._setZWarpNode();
		
		this.replaceTransformNode(oldNode, this.deformation)
	}
  }
}

let widgetsDescription = [
	  {
		type: 'inputData',
		inputDataSize: 1,
		info: 'Input vector used for deformation'
	  },
	  {
		name: 'ZWarp Factor',
		type: 'FloatSlider',
		info: 'Factor used for Zwarp effect',
		parameter: 'defFactor',
		min: -1000,
		max: 1000,
		param : true
	  } ] ;

module.exports = {block:myBlock1,
				blockType:'myBlock1',
				widgetsDescription:widgetsDescription};
