
let SciviJS_GUI=require('scivijs_gui');
let SciviJS=SciviJS_GUI.SciviJS;
let PlugInBlock = SciviJS.PlugInBlock;

class myPoints extends PlugInBlock {

  /**
   * Constructor for yourBlock block
   * @param myFirstParameter : ...
   * @param ...
   * **/
  validate(parent){
	  let valid = true;
    // Check if one parent is a VectorField or a Points
    while (parent !== undefined) {
      if (parent.blockType === 'VectorField' ||
          parent.blockType === 'Points') { return false; }
      parent = parent.parentBlock;
    }
   // console.log(valid);
    return valid;
  }
   
  constructor (parentBlock, pointsSize = 3, mode = 'volume') {
    // Only initialize setters (maybe getters if you want), this,
    // and your custom parameters


    super(parentBlock);

    this._pointsSize = pointsSize;
    this._mode = mode;
  };

  /**
   * Method that initialize your node structures and geometries
   * **/
  _process () {
    // Compute geometry
    this._pointsBufferGeometry = new THREE.BufferGeometry();
    this.updateGeometry();

    // Get current material
    this._pointsMaterial = this.getCurrentMaterial();

    // Set mesh
    let points = new THREE.Points(
      this._pointsBufferGeometry,
      this._pointsMaterial
    );

    // Remove all other meshes from scene
    this.removeMeshes();

    // Add new point mesh
    this.addMesh(points);

    // Set point size
    this._sizeNode = new THREE.FloatNode(this._pointsSize);
    super.addPointSizeNode(this._sizeNode);
  };

  /**
   * Update geometry when needed
   * **/
  _updateGeometry () {
// Initialize geometry
    let coordArray = this.parentBlock.coordArray;
    let facesArray = this.parentBlock.facesArray;

    if (facesArray === undefined && this.mode == 'surface') {
      throw new Error('Cannot compute Points in surface mode without ' +
        'faces indices.');
    }

    // Delete duplicate indices
    let surfaceIndexes = new Set(facesArray);
    surfaceIndexes = Array.from(surfaceIndexes);

    let vertex, x, y, z;

    let nbVertices;
    switch (this._mode) {
		
      case 'surface':
        nbVertices = surfaceIndexes.length;
        break;
      case 'volume':
        nbVertices = coordArray.length / 3;
        break;
      default:
        nbVertices = coordArray.length / 3;
        this._mode = 'volume';
        break;
    }

    // Compute the number of displayed points
    this._nbPoints =  nbVertices;

    let pointsCoordArray = [];

    // Get input arrays and create pointsDataArrays
    let inputDataArrays = [];
    Object.values(this.data).forEach((data) => {
      Object.keys(data).forEach((componentName) => {
        if (componentName !== 'Magnitude') {
          inputDataArrays.push(data[componentName].array);
        }
      });
    });
    let len = inputDataArrays.length;
    let pointsDataArrays = new Array(len);
    while (len--) {
      pointsDataArrays[len] = [];
    }

    for (let i = 0; i < this._nbPoints; i++) {
          switch (this._mode) {
            case 'volume':
              vertex = i;
              break;
            case 'surface':
              vertex = surfaceIndexes[i];
              break;
          }
          break;
      

      x = coordArray[vertex * 3];
      y = coordArray[vertex * 3 + 1];
      z = coordArray[vertex * 3 + 2];

      // Add point
      pointsCoordArray.push(x);
      pointsCoordArray.push(y);
      pointsCoordArray.push(z);

      // Fill vectorsDataArrays
      pointsDataArrays.forEach((dataArray, dataIndex) => {
        let dataValue = inputDataArrays[dataIndex][vertex];
        dataArray.push(dataValue);
      });
    }

    let coordAttributes = new THREE.BufferAttribute(
      new Float32Array(pointsCoordArray),
      3
    );
    this._pointsBufferGeometry.removeAttribute('position');
    this._pointsBufferGeometry.addAttribute(
      'position',
      coordAttributes
    );

    // One buffer per data
    let dataIndex = 0;
    Object.values(this.data).forEach((data) => {
      Object.values(data).forEach((component) => {
        if (!component.shaderName.endsWith('Magnitude')) {
          let bufferArray = new Float32Array(
            pointsDataArrays[dataIndex]);
          let dataAttribute = new THREE.BufferAttribute(
            bufferArray,
            1
          );
          this._pointsBufferGeometry.removeAttribute(
            component.shaderName);
          this._pointsBufferGeometry.addAttribute(
            component.shaderName, dataAttribute);

          dataIndex++;
        }
      });
    });

    this.facesArray = undefined;
    this.tetraArray = undefined;
  

  };
};

let widgetsDescription= [
  {
    name: 'Mode',
    type: 'DropDown',
    info: 'Mode used for points, can be volume or surface',
    parameter: 'mode',
    enum: {
      volume: 'volume',
      surface: 'surface'
    }
  },
  {
    name: 'Points Size',
    type: 'FloatSlider',
    info: 'Size of points in pixels. Minimum value is 0.1, maximum value is 50',
    parameter: 'pointsSize',
    min: 0.1,
    max: 50
  },
  {
    name: 'Points Number',
    type: 'FloatSlider',
    info: 'Percentage of displayed points. If 100% maximum number of points is displayed, if 0% no point is displayed.',
    parameter: 'pcPoints',
    min: 0,
    max: 1,
    percentage: true
  }
];



module.exports = {block:myPoints,
				blockType:'myPoints',
				widgetsDescription:widgetsDescription};
